@@ -199,8 +199,12 @@ jack_error (const char *fmt, ...)
 void 
 default_jack_error_callback (const char *desc)
 {
+#ifdef DEBUG_ENABLED
+	DEBUG("%s", desc);
+#else
 	fprintf(stderr, "%s\n", desc);
 	fflush(stderr);
+#endif
 }
 
 void 
@@ -952,6 +956,8 @@ jack_client_open_aux (const char *client_name,
 	jack_port_type_id_t ptid;
 	jack_status_t my_status;
 
+	jack_messagebuffer_init ();
+	
 	if (status == NULL)		/* no status from caller? */
 		status = &my_status;	/* use local status word */
 	*status = 0;
@@ -1099,10 +1105,10 @@ jack_client_open_aux (const char *client_name,
 jack_client_t* jack_client_open(const char* ext_client_name, jack_options_t options, jack_status_t* status, ...)
 {
 	va_list ap;
-    va_start(ap, status);
-    jack_client_t* res = jack_client_open_aux(ext_client_name, options, status, ap);
-    va_end(ap);
-    return res;
+	va_start(ap, status);
+	jack_client_t* res = jack_client_open_aux(ext_client_name, options, status, ap);
+	va_end(ap);
+	return res;
 }
 
 jack_client_t *
@@ -1201,7 +1207,7 @@ jack_internal_client_close (const char *client_name)
 		jack_error ("cannot deliver ClientUnload request to JACK "
 			    "server.");
 	}
-	
+
 	/* no response to this request */
 	
 	close (fd);
@@ -1301,6 +1307,8 @@ jack_client_process_events (jack_client_t* client)
 	JSList *node;
 	jack_port_t* port;
 
+	DEBUG ("process events");
+
 	if (client->pollfd[EVENT_POLL_INDEX].revents & POLLIN) {
 		
 		DEBUG ("client receives an event, "
@@ -1683,14 +1691,26 @@ jack_client_thread_aux (void *arg)
 	/* wait for first wakeup from server */
 
 	if (jack_thread_first_wait (client) == control->nframes) {
+
 		/* now run till we're done */
+
 		if (control->process) {
+
 			/* run process callback, then wait... ad-infinitum */
-			while (jack_thread_wait (client, 
-						 control->process (control->nframes, 
-								   control->process_arg)) ==
-			       control->nframes)
-				;
+
+			while (1) {
+				DEBUG("client calls process()");
+				int status = (control->process (control->nframes, 
+								control->process_arg) ==
+					      control->nframes);
+				control->state = Finished;
+				DEBUG("client leaves process(), re-enters wait");
+				if (!jack_thread_wait (client, status)) {
+					break;
+				}
+				DEBUG("client done with wait");
+			}
+
 		} else {
 			/* no process handling but still need to process events */
 			while (jack_thread_wait (client, 0) == control->nframes)
@@ -2081,6 +2101,7 @@ jack_client_close_aux (jack_client_t *client)
 	}
 	jack_slist_free (client->ports_ext);
 	jack_client_free (client);
+	jack_messagebuffer_exit ();
 
 	return rc;
 }
